@@ -28,6 +28,52 @@
 #' Harrell, F. E., Lee, K. L., and Mark, D. B. (1996). Tutorial in biostatistics multivariable prognostic models: issues in developing models,
 #' evaluating assumptions and adequacy, and measuring and reducing errors. Statistics in medicine, 15, 361-387.
 #' doi:10.1002/(SICI)1097-0258(19960229)15:4<361::AID-SIM168>3.0.CO;2-4
+#' @details Estimates of various quantities conditional on subgroups and treatment statuses are provided and displayed
+#' via the \code{\link[personalized]{print.subgroup_validated}} function:
+#' \enumerate{
+#' \item{"Conditional expected outcomes"}{ The first results shown when printing
+#' a \code{subgroup_validated} object are estimates of the expected outcomes conditional on
+#' the estimated subgroups (i.e. which subgroup is 'recommended' by the model) and conditional
+#' on treatment/intervention status. If there are two total treatment options, this results in a 2x2 table
+#' of expected conditional outcomes. }
+#' \item{"Treatment effects conditional on subgroups"}{ The second results shown when printing
+#' a \code{subgroup_validated} object are estimates of the expected outcomes conditional on
+#' the estimated subgroups. If the treatment takes levels \eqn{j \in \{1, \dots, K\}}, a total of \eqn{K}
+#' conditional treatment effects will be shown. For example, of the outcome is continuous, the
+#' \eqn{j}th conditional treatment effect is defined as \eqn{E(Y|Trt = j, Subgroup=j) - E(Y|Trt = j, Subgroup =/= j)}},
+#' where \eqn{Subgroup=j} if treatment \eqn{j} is recommended, i.e. treatment \eqn{j} results in the largest/best
+#' expected potential outcomes given the fitted model.
+#' \item{"Overall treatment effect conditional on subgroups "}{ The third quantity displayed shows the overall improvement
+#' in outcomes resulting from all treatment recommendations. This is essentially an average over all of the
+#' conditional treatment effects weighted by the proportion of the population recommended each respective
+#' treatment level.}
+#' }
+#' @return An object of class \code{"subgroup_validated"}
+#' \item{avg.results}{Estimates of average conditional treatment effects when
+#' subgroups are determined based on the provided cutoff value for the benefit score. For example,
+#' if \code{cutoff = 0} and there is a treatment and control only, then the treatment is
+#' recommended if the benefit score is greater than 0.}
+#' \item{se.results}{Standard errors of the estimates from \code{avg.estimates}}
+#' \item{boot.results}{Contains the individual results for each replication. \code{avg.results} is comprised
+#' of averages of the values from \code{boot.results}}
+#' \item{avg.quantile.results}{Estimates of average conditional treatment effects when
+#' subgroups are determined based on different quntile cutoff values for the benefit score. For example,
+#' if \code{benefit.score.quantiles = 0.75} and there is a treatment and control only, then the treatment is
+#' recommended if the benefit score is greater than the 75th upper quantile of all benefit scores. If multiple quantile
+#' values are provided, e.g. \code{benefit.score.quantiles = c(0.15, 0.5, 0.85)}, then results will be provided
+#' for all quantile levels.}
+#' \item{se.quantile.results}{Standard errors corresponding to \code{avg.quantile.results}}
+#' \item{boot.results.quantiles}{Contains the individual results for each replication. \code{avg.quantile.results} is comprised
+#' of averages of the values from \code{boot.results.quantiles}}
+#' \item{family}{Family of the outcome. For example, \code{"gaussian"} for continuous outcomes}
+#' \item{method}{Method used for subgroup identification model. Weighting or A-learning}
+#' \item{n.trts}{The number of treatment levels}
+#' \item{comparison.trts}{All treatment levels other than the reference level}
+#' \item{reference.trt}{The reference level for the treatment. This should usually be the control group/level}
+#' \item{larger.outcome.better}{If larger outcomes are preferred for this model}
+#' \item{cutpoint}{Benefit score cutoff value used for determining subgroups}
+#' \item{val.method}{Method used for validation}
+#' \item{iterations}{Number of replications used in the validation process}
 #' @importFrom stats predict sd
 #' @import foreach
 #' @examples
@@ -90,22 +136,13 @@
 #'
 #' y.test <- drop(xbeta.test) + rnorm(10 * n.obs, sd = 2)
 #'
-#' valmod <- validate.subgroup(subgrp.model, B = 10,
+#' valmod <- validate.subgroup(subgrp.model, B = 3,
 #'                             method = "training_test",
 #'                             train.fraction = 0.75)
 #' valmod
 #'
 #' print(valmod, which.quant = c(4, 5))
 #'
-#' bene.score.test <- subgrp.model$predict(x.test)
-#'
-#' mean(y.test[bene.score.test > 0 & trt01.test == 1]) -
-#'        mean(y.test[bene.score.test > 0 & trt01.test == 0])
-#' mean(y.test[bene.score.test <= 0 & trt01.test == 0]) -
-#'        mean(y.test[bene.score.test <= 0 & trt01.test == 1])
-#'
-#' quantile(valmod$boot.results[[1]][,1], c(0.025, 0.975))
-#' quantile(valmod$boot.results[[1]][,2], c(0.025, 0.975))
 #' @export
 validate.subgroup <- function(model,
                               B              = 50L,
@@ -215,11 +252,12 @@ validate.subgroup <- function(model,
 
     if (parallel)
     {
-        comb <- function(x, ...)
-        {
-            lapply(seq_along(x),
-                   function(i) c(x[[i]], lapply(list(...), function(y) y[[i]])))
-        }
+        ## no longer needed:
+        #comb <- function(x, ...)
+        #{
+        #    lapply(seq_along(x),
+        #           function(i) c(x[[i]], lapply(list(...), function(y) y[[i]])))
+        #}
 
         outlist <- foreach(i = seq(B) ) %dopar% #, .combine = "comb", .multicombine = TRUE, .init=list(list(), list(), list(), list(), list()))
                            {
@@ -624,6 +662,8 @@ validate.subgroup <- function(model,
     names(boot.list)[c(1:3, 5)] <- names(summary.stats)
     names(boot.list)[4] <- "coefficients"
 
+    colnames(boot.list[[1]]) <- names(model$subgroup.trt.effects$subgroup.effects)
+
     summary.stats.quantile <- summary.stats.quantile.se <- vector(mode = "list", length = n.quantiles)
     for (q in 1:n.quantiles)
     {
@@ -650,6 +690,8 @@ validate.subgroup <- function(model,
         names(boot.list.quantiles[[q]]) <- 1:length(names(boot.list))
         names(boot.list.quantiles[[q]])[c(1:3, 5)] <- names(summary.stats)
         names(boot.list.quantiles[[q]])[4] <- "coefficients"
+
+        colnames(boot.list.quantiles[[q]][[1]]) <- names(model$subgroup.trt.effects$subgroup.effects)
     }
 
     names(summary.stats.quantile) <- names(summary.stats.quantile.se) <-
@@ -675,6 +717,11 @@ validate.subgroup <- function(model,
                 family       = model$family,     # model family
                 loss         = model$loss,       # model loss
                 method       = model$method,     # subgroup method (weighting vs a-learning)
+                n.trts       = model$n.trts,
+                comparison.trts       = model$comparison.trts,
+                reference.trt         = model$reference.trt,
+                larger.outcome.better = model$larger.outcome.better,
+                cutpoint              = model$cutpoint,
                 val.method   = method,
                 iterations   = B)
     class(ret) <- "subgroup_validated"
